#include "PE_Types.h"
#include "trcKernel.h"

#define traceTASK_INCREMENT_TICK(xTickCount) \
  { \
    extern uint32_t uiTraceTickCount; \
    uiTraceTickCount = xTickCount; \
  }

/* Called after a task has been selected to run.  pxCurrentTCB holds a pointer
 * to the task control block of the selected task.
 */
#define traceTASK_SWITCHED_IN() \
  vTraceStoreTaskswitch();

/* Called before a task has been selected to run.  pxCurrentTCB holds a pointer
 * to the task control block of the task being switched out.
 */
/* traceTASK_SWITCHED_OUT not implemented yet */

/* Task is about to block because it cannot read from a
 * queue/mutex/semaphore.  pxQueue is a pointer to the queue/mutex/semaphore
 * upon which the read was attempted.  pxCurrentTCB points to the TCB of the
 * task that attempted the read.
 */
#define traceBLOCKING_ON_QUEUE_RECEIVE( pxQueue ) \
  portENTER_CRITICAL(); \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_BLOCK_ON_RECEIVE + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber); \
  if (TraceObjectClassTable[pxQueue->ucQueueType] != TRACE_CLASS_MUTEX){\
    extern volatile void * volatile pxCurrentTCB; \
    vTraceSetTaskInstanceFinished((objectHandleType)uxTaskGetTaskNumber((xTaskHandle)pxCurrentTCB)); \
  }\
  portEXIT_CRITICAL();

/* Task is about to block because it cannot write to a
 * queue/mutex/semaphore.  pxQueue is a pointer to the queue/mutex/semaphore
 * upon which the write was attempted.  pxCurrentTCB points to the TCB of the
 * task that attempted the write.
 */
#define traceBLOCKING_ON_QUEUE_SEND( pxQueue ) \
  portENTER_CRITICAL();\
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_BLOCK_ON_SEND + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber); \
  portEXIT_CRITICAL();

#define traceQUEUE_CREATE( pxNewQueue )\
  portENTER_CRITICAL(); \
  pxNewQueue->ucQueueNumber = xTraceGetObjectHandle(TraceObjectClassTable[pxNewQueue->ucQueueType]);\
  vTraceSetObjectState(TraceObjectClassTable[pxNewQueue->ucQueueType], pxNewQueue->ucQueueNumber, 0); \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_CREATE + TraceObjectClassTable[pxNewQueue->ucQueueType]), pxNewQueue->ucQueueNumber); \
  portEXIT_CRITICAL();

#define traceQUEUE_CREATE_FAILED( queueType ) \
  portENTER_CRITICAL();\
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_FAILED_CREATE + TraceObjectClassTable[queueType]), (objectHandleType)0); \
  portEXIT_CRITICAL();

#define traceCREATE_MUTEX( pxNewQueue ) \
  portENTER_CRITICAL();\
  pxNewQueue->ucQueueNumber = xTraceGetObjectHandle(TRACE_CLASS_MUTEX); \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_CREATE + TraceObjectClassTable[pxNewQueue->ucQueueType]), pxNewQueue->ucQueueNumber); \
  vTraceSetObjectState(TRACE_CLASS_MUTEX, pxNewQueue->ucQueueNumber, 0); \
  portEXIT_CRITICAL();

#define traceCREATE_MUTEX_FAILED() \
  portENTER_CRITICAL();\
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_FAILED_CREATE + TRACE_CLASS_MUTEX), 0);\
  portEXIT_CRITICAL();

/* traceGIVE_MUTEX_RECURSIVE not implemented yet */

#define traceGIVE_MUTEX_RECURSIVE_FAILED( pxMutex ) \
  portENTER_CRITICAL();\
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_FAILED_SEND + TRACE_CLASS_MUTEX), pxMutex->ucQueueNumber); \
  portEXIT_CRITICAL();

/* traceTAKE_MUTEX_RECURSIVE not implemented yet */

/* traceCREATE_COUNTING_SEMAPHORE not implemented yet */

/* traceCREATE_COUNTING_SEMAPHORE_FAILED not implemented yet */

#define traceQUEUE_SEND( pxQueue ) \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_SEND + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber); \
  if (TraceObjectClassTable[pxQueue->ucQueueType] == TRACE_CLASS_MUTEX){\
    vTraceSetObjectState(TraceObjectClassTable[pxQueue->ucQueueType], (uint8_t)pxQueue->ucQueueNumber, (uint8_t)0); \
  }else{\
    vTraceSetObjectState(TraceObjectClassTable[pxQueue->ucQueueType], (uint8_t)pxQueue->ucQueueNumber, (uint8_t)(pxQueue->uxMessagesWaiting + 1)); \
  }

#define traceQUEUE_SEND_FAILED( pxQueue ) \
  portENTER_CRITICAL();\
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_FAILED_SEND + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber); \
  portEXIT_CRITICAL();

#define traceQUEUE_RECEIVE( pxQueue ) \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_RECEIVE + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber); \
  if (TraceObjectClassTable[pxQueue->ucQueueType] == TRACE_CLASS_MUTEX){\
    extern volatile void * volatile pxCurrentTCB; \
    vTraceSetObjectState(TraceObjectClassTable[pxQueue->ucQueueType], (objectHandleType)pxQueue->ucQueueNumber, (objectHandleType)uxTaskGetTaskNumber((xTaskHandle)pxCurrentTCB)); /*For mutex, store the new owner rather than queue length */ \
  }else{\
    vTraceSetObjectState(TraceObjectClassTable[pxQueue->ucQueueType], (objectHandleType)pxQueue->ucQueueNumber, (uint8_t)(pxQueue->uxMessagesWaiting - 1)); \
  }

#define traceQUEUE_RECEIVE_FAILED( pxQueue ) \
  portENTER_CRITICAL(); \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_FAILED_RECEIVE + TraceObjectClassTable[pxQueue->ucQueueType]),  pxQueue->ucQueueNumber); \
  portEXIT_CRITICAL();

#define traceQUEUE_PEEK( pxQueue ) \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_PEEK + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber);

#define traceQUEUE_SEND_FROM_ISR( pxQueue ) \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_SEND_FROM_ISR + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber); \
  vTraceSetObjectState(TRACE_CLASS_QUEUE, (objectHandleType)pxQueue->ucQueueNumber, (uint8_t)(pxQueue->uxMessagesWaiting + 1));

#define traceQUEUE_RECEIVE_FROM_ISR( pxQueue )  \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_RECEIVE_FROM_ISR + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber); \
  vTraceSetObjectState(TRACE_CLASS_QUEUE, (objectHandleType)pxQueue->ucQueueNumber, (uint8_t)(pxQueue->uxMessagesWaiting - 1));

#define traceQUEUE_SEND_FROM_ISR_FAILED( pxQueue ) \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_FAILED_SEND_FROM_ISR + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber);

#define traceQUEUE_RECEIVE_FROM_ISR_FAILED( pxQueue ) \
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_FAILED_RECEIVE_FROM_ISR + TraceObjectClassTable[pxQueue->ucQueueType]), pxQueue->ucQueueNumber);

#if (INCLUDE_OBJECT_DELETE == 1)
#define traceQUEUE_DELETE( pxQueue ) \
{ \
  portENTER_CRITICAL();\
  vTraceStoreKernelCall(EVENTGROUP_DELETE + TraceObjectClassTable[pxQueue->ucQueueType], pxQueue->ucQueueNumber); \
  vTraceStoreObjectNameOnCloseEvent((objectHandleType)pxQueue->ucQueueNumber, TraceObjectClassTable[pxQueue->ucQueueType]); \
  vTraceStoreObjectPropertiesOnCloseEvent((objectHandleType)pxQueue->ucQueueNumber, TraceObjectClassTable[pxQueue->ucQueueType]); \
  if (TraceObjectClassTable[pxQueue->ucQueueType] == TRACE_CLASS_MUTEX){ \
    vTraceSetObjectState(TraceObjectClassTable[pxQueue->ucQueueType], (objectHandleType)pxQueue->ucQueueNumber, (objectHandleType)uxTaskGetTaskNumber((xTaskHandle)pxQueue->pxMutexHolder)); \
  }else{ \
    vTraceSetObjectState(TraceObjectClassTable[pxQueue->ucQueueType], (objectHandleType)pxQueue->ucQueueNumber, (uint8_t)uxQueueMessagesWaiting(pxQueue)); \
  } \
  vTraceFreeObjectHandle(TraceObjectClassTable[pxQueue->ucQueueType], (objectHandleType)pxQueue->ucQueueNumber); \
  portEXIT_CRITICAL();\
}
#endif

#define traceTASK_CREATE( pxNewTCB ) \
  if (pxNewTCB != NULL){ \
    pxNewTCB->uxTaskNumber = xTraceGetObjectHandle(TRACE_CLASS_TASK); \
    vTraceSetObjectName(TRACE_CLASS_TASK, (objectHandleType)pxNewTCB->uxTaskNumber, (char*)pxNewTCB->pcTaskName); \
    vTraceSetPriorityProperty(TRACE_CLASS_TASK, (objectHandleType)pxNewTCB->uxTaskNumber, (uint8_t)pxNewTCB->uxPriority); \
    vTraceStoreKernelCall((uint32_t)(EVENTGROUP_CREATE + TRACE_CLASS_TASK), (objectHandleType)pxNewTCB->uxTaskNumber);\
  }

#define traceTASK_CREATE_FAILED() \
  portENTER_CRITICAL();\
  vTraceStoreKernelCall((uint32_t)(EVENTGROUP_FAILED_CREATE + TRACE_CLASS_TASK), 0); \
  portEXIT_CRITICAL();

#if INCLUDE_vTaskDelete
#define traceTASK_DELETE( pxTaskToDelete ) \
  vTraceStoreKernelCall(EVENTGROUP_DELETE + TRACE_CLASS_TASK, pxTaskToDelete->uxTaskNumber); \
  vTraceStoreObjectNameOnCloseEvent((objectHandleType)pxTaskToDelete->uxTaskNumber, TRACE_CLASS_TASK); \
  vTraceStoreObjectPropertiesOnCloseEvent((objectHandleType)pxTaskToDelete->uxTaskNumber, TRACE_CLASS_TASK); \
  vTraceSetPriorityProperty(TRACE_CLASS_TASK, (objectHandleType)pxTaskToDelete->uxTaskNumber, (uint8_t)pxTaskToDelete->uxPriority); \
  vTraceSetObjectState(TRACE_CLASS_TASK, (objectHandleType)pxTaskToDelete->uxTaskNumber, TASK_STATE_INSTANCE_NOT_ACTIVE); \
  vTraceFreeObjectHandle(TRACE_CLASS_TASK, (objectHandleType)pxTaskToDelete->uxTaskNumber);

#endif
#define traceTASK_DELAY_UNTIL() \
  portENTER_CRITICAL(); \
  vTraceStoreKernelCallWithNumericParamOnly(TASK_DELAY_UNTIL, (uint16_t)xTimeToWake); \
  vTraceSetTaskInstanceFinished((uint8_t)pxCurrentTCB->uxTaskNumber); \
  portEXIT_CRITICAL();

#define traceTASK_DELAY() \
  portENTER_CRITICAL(); \
  vTraceStoreKernelCallWithNumericParamOnly(TASK_DELAY, (uint16_t)xTicksToDelay);\
  vTraceSetTaskInstanceFinished((uint8_t)pxCurrentTCB->uxTaskNumber);\
  portEXIT_CRITICAL();

#define traceTASK_PRIORITY_SET( pxTask, uxNewPriority ) \
  vTraceStoreKernelCallWithParam(TASK_PRIORITY_SET, pxTask->uxTaskNumber, uiTraceGetPriorityProperty(TRACE_CLASS_TASK, (uint8_t)pxTask->uxTaskNumber));\
  vTraceSetPriorityProperty( TRACE_CLASS_TASK, (uint8_t)pxTask->uxTaskNumber, (uint8_t)uxNewPriority);

#define traceTASK_SUSPEND( pxTaskToSuspend ) \
  vTraceStoreKernelCall(TASK_SUSPEND, pxTaskToSuspend->uxTaskNumber); \
  vTraceSetTaskInstanceFinished((uint8_t)pxTaskToSuspend->uxTaskNumber);

#define traceTASK_RESUME( pxTaskToResume ) \
  vTraceStoreKernelCall(TASK_RESUME, pxTaskToResume->uxTaskNumber);

#define traceTASK_RESUME_FROM_ISR( pxTaskToResume ) \
  vTraceStoreKernelCall(TASK_RESUME_FROM_ISR, pxTaskToResume->uxTaskNumber);

#define traceTASK_PRIORITY_INHERIT( pxTask, uxNewPriority ) \
  vTraceStoreKernelCallWithParam(TASK_PRIORITY_INHERIT, pxTask->uxTaskNumber, uiTraceGetPriorityProperty(TRACE_CLASS_TASK, (uint8_t)pxTask->uxTaskNumber));\
  vTraceSetPriorityProperty( TRACE_CLASS_TASK, (uint8_t)pxTask->uxTaskNumber, (uint8_t)uxNewPriority );

#define traceTASK_PRIORITY_DISINHERIT( pxTask, uxNewPriority ) \
  vTraceStoreKernelCallWithParam(TASK_PRIORITY_DISINHERIT, pxTask->uxTaskNumber, uiTraceGetPriorityProperty(TRACE_CLASS_TASK, (uint8_t)pxTask->uxTaskNumber));\
  vTraceSetPriorityProperty( TRACE_CLASS_TASK, (uint8_t)pxTask->uxTaskNumber, (uint8_t)uxNewPriority );

